/* 

Copyright (C) 2003-2004 Yves Schmid (www.garagecube.com)

Author(s): Yves Schmid

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     GNU General Public License for more details.

You should have received a copy of the GNU General Public Licensealong with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#import "CosmicMirror.h"
#import "CosmicOpenGLView.h"

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#include <OpenGL/OpenGL.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>

#include "OMediaGLRenderer.h"
#include "OMediaCanvasElement.h"
#include "OMediaViewPort.h"

class CosmicGLMirrorRenderTarget : public OMediaPureGLRenderTarget
{
    public:

    NSOpenGLContext	*glContext;
    
    CosmicGLMirrorRenderTarget() {glContext=NULL;}

    virtual void set_context(void)
    {
        if (glContext) [glContext makeCurrentContext];
    }    
};


@implementation CosmicMirror



static CosmicMirror *mainMirror;

- (id) initWithSize: (NSSize) wsize
{
	CGDisplayCount num_displays;
	CGDirectDisplayID *display_ids;
	
	if (CGGetActiveDisplayList(0, NULL, &num_displays) != CGDisplayNoErr || num_displays <= 1) return nil;

	display_ids = new CGDirectDisplayID[num_displays];

	if (CGGetActiveDisplayList(num_displays, display_ids, &num_displays) != CGDisplayNoErr) return nil;

	if (CGDisplayIsCaptured(display_ids[1])) return nil;

	if (CGDisplayCapture(display_ids[1]) != CGDisplayNoErr) return nil;
	
	CGRect bounds = CGDisplayBounds(display_ids[1]);

	width = (int)bounds.size.width;
	height = (int)bounds.size.height;
	
	[super initWithContentRect:NSMakeRect(bounds.origin.x,bounds.origin.y,bounds.size.width,bounds.size.height) styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];

	NSOpenGLPixelFormatAttribute attribs[] =
	{
		NSOpenGLPFANoRecovery,
		NSOpenGLPFAAccelerated,
		NSOpenGLPFADoubleBuffer,
		NSOpenGLPFAColorSize, (NSOpenGLPixelFormatAttribute) 24,
		NSOpenGLPFAAlphaSize, (NSOpenGLPixelFormatAttribute) 8,
		NSOpenGLPFADepthSize, (NSOpenGLPixelFormatAttribute) 0,
		NSOpenGLPFAStencilSize, (NSOpenGLPixelFormatAttribute) 0,
		NSOpenGLPFAAccumSize, (NSOpenGLPixelFormatAttribute) 0,
		NSOpenGLPFAFullScreen,
		NSOpenGLPFAScreenMask, (NSOpenGLPixelFormatAttribute) CGDisplayIDToOpenGLDisplayMask(display_ids[1]),
		(NSOpenGLPixelFormatAttribute) 0
	};

	NSOpenGLPixelFormat* fmt = [[[NSOpenGLPixelFormat alloc] initWithAttributes: attribs] autorelease];
	ctx = [[NSOpenGLContext alloc] initWithFormat: fmt shareContext: nil];
		
	[ctx setFullScreen];
	[ctx makeCurrentContext];

    long params[] = { 1};
	CGLSetParameter(CGLGetCurrentContext(),  kCGLCPSwapInterval, params);
	
	delete [] display_ids;

	renderTarget = new CosmicGLMirrorRenderTarget();
	renderTarget->glContext = ctx;
	
	return self;
}

-(void)dealloc
{
	delete renderTarget;
	[super dealloc];
}

+(void)startMirroring
{
	if (mainMirror) return;
	mainMirror = [[CosmicMirror alloc] initWithSize:NSMakeSize(800,600)];
}

+(void)stopMirroring
{
	CosmicMirror	*mirror = mainMirror;

	if (mirror) 
	{ 
		[mirror->mirroredView lock];
		mainMirror = NULL;
		[mirror->ctx makeCurrentContext];
		if (mirror->mirroredView) [mirror->mirroredView flushCanvasGLTexture];
		[mirror->ctx release];
		[mirror->mirroredView unlock];
		[mirror release];

		
		// Release display

		CGDisplayCount num_displays;
		CGDirectDisplayID *display_ids;
		
		if (CGGetActiveDisplayList(0, NULL, &num_displays) != CGDisplayNoErr) return;

		display_ids = new CGDirectDisplayID[num_displays];

		CGGetActiveDisplayList(num_displays, display_ids, &num_displays);

		CGDisplayRelease(display_ids[1]);
		
		delete [] display_ids;
	}
	
}

+(CosmicMirror*)getMirror
{
	return mainMirror;
}

+(void)setViewToMirror:(CosmicOpenGLView*)view
{
	[CosmicMirror startMirroring];
	if (!mainMirror) return;
	if (mainMirror->mirroredView) [mainMirror->mirroredView flushCanvasGLTexture];
	mainMirror->mirroredView = view;
}

+(CosmicOpenGLView*)getViewToMirror
{
	if (!mainMirror) return NULL;
	return mainMirror->mirroredView;
}

+(void)mirrorRenderView:(CosmicOpenGLView*)view
{
	if (!mainMirror) return;
	if (view!=mainMirror->mirroredView || !view) return;
	if (mainMirror->mirroredView->renderTarget==NULL) return;

    OMediaRect	vp_bounds;

    [mainMirror->ctx makeCurrentContext];

	mainMirror->mirroredView->crossCanvasElement->hide();
	mainMirror->mirroredView->bufferCanvasElement->hide();

    vp_bounds.set(0,0, (long) mainMirror->width, (long) mainMirror->height);
    mainMirror->mirroredView->viewport->setbounds(&vp_bounds);
	
	float oldz = mainMirror->mirroredView->viewport->getz();

	mainMirror->mirroredView->viewport->setz(oldz+50);

    mainMirror->renderTarget->render_viewport(mainMirror->mirroredView->viewport);

	mainMirror->mirroredView->viewport->setz(oldz);
	
    // Swap buffer to screen
    [mainMirror->ctx flushBuffer];
}


@end
